"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.performDcqlQuery = void 0;
const dcql_credential_query_result_js_1 = require("../dcql-query-result/dcql-credential-query-result.js");
const performDcqlQuery = (dcqlQuery, credentials) => {
    const credentialQueriesResults = Object.fromEntries(dcqlQuery.credentials.map(credentialQuery => [
        credentialQuery.id,
        (0, dcql_credential_query_result_js_1.queryCredentialFromCredentialQuery)(credentialQuery, credentials),
    ]));
    const credentialMatches = Object.fromEntries(Object.entries(credentialQueriesResults).map(([key, credentialQueryResult]) => {
        // Find the best match for each credential query
        let bestMatch = undefined;
        for (const credentialParseResult of credentialQueryResult) {
            const bestMatchForCredential = credentialParseResult.find(result => result?.success == true);
            if (!bestMatch) {
                bestMatch = bestMatchForCredential;
                continue;
            }
            if (bestMatchForCredential &&
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                bestMatchForCredential.claim_set_index < bestMatch.claim_set_index) {
                bestMatch = bestMatchForCredential;
            }
        }
        return [
            key,
            bestMatch
                ? { ...bestMatch, all: credentialQueryResult }
                : // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    { ...credentialQueryResult[0][0], all: credentialQueryResult },
        ];
    }));
    const credentialSetResults = dcqlQuery.credential_sets?.map(set => {
        const matchingOptions = set.options.filter(option => option.every(credentialQueryId => credentialMatches[credentialQueryId]?.success));
        return {
            ...set,
            matching_options: matchingOptions.length > 0
                ? matchingOptions
                : undefined,
        };
    });
    const dqclQueryMatched = credentialSetResults
        ? credentialSetResults.every(set => !set.required || set.matching_options)
        : Object.values(credentialMatches).every(query => query.success);
    return {
        ...dcqlQuery,
        canBeSatisfied: dqclQueryMatched,
        credential_matches: credentialMatches,
        credential_sets: credentialSetResults,
    };
};
exports.performDcqlQuery = performDcqlQuery;
//# sourceMappingURL=dcql-query.js.map